@@ -23,21 +23,25 @@
 #define __PANGO_ITEM_H__
 
 #include <pango/pango-types.h>
-#include <pango/pango-item.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 typedef struct _PangoAnalysis PangoAnalysis;
 typedef struct _PangoItem PangoItem;
 
+/* TODO: if more flags are needed, turn this into a real PangoAnalysisFlags enum */
+#define PANGO_ANALYSIS_FLAG_CENTERED_BASELINE (1 << 0)
+
 struct _PangoAnalysis
 {
   PangoEngineShape *shape_engine;
   PangoEngineLang  *lang_engine;
   PangoFont *font;
+
   guint8 level;
+  guint8 gravity; /* PangoGravity */
+  guint8 flags;
+
   PangoLanguage *language;
   GSList *extra_attrs;
 };
@@ -50,6 +54,10 @@ struct _PangoItem
   PangoAnalysis analysis;
 };
 
+#define PANGO_TYPE_ITEM (pango_item_get_type ())
+
+GType pango_item_get_type (void) G_GNUC_CONST;
+
 PangoItem *pango_item_new   (void);
 PangoItem *pango_item_copy  (PangoItem  *item);
 void       pango_item_free  (PangoItem  *item);
@@ -57,8 +65,6 @@ PangoItem *pango_item_split (PangoItem  *orig,
                              int         split_index,
                              int         split_offset);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __PANGO_ITEM_H__ */
