@@ -19,13 +19,14 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
 
-#include <pango/pango-attributes.h>
-#include <pango/pango-font.h>
-#include <pango/pango-utils.h>
+#include "pango-attributes.h"
+#include "pango-font.h"
+#include "pango-impl-utils.h"
 
 /* FIXME */
 #define _(x) x
@@ -190,7 +191,7 @@ open_tag_free (OpenTag *ot)
 {
   g_slist_foreach (ot->attrs, (GFunc) pango_attribute_destroy, NULL);
   g_slist_free (ot->attrs);
-  g_free (ot);
+  g_slice_free (OpenTag, ot);
 }
 
 static void
@@ -225,7 +226,7 @@ markup_data_open_tag (MarkupData   *md)
   if (md->tag_stack)
     parent = md->tag_stack->data;
   
-  ot = g_new (OpenTag, 1);
+  ot = g_slice_new (OpenTag);
   ot->attrs = NULL;
   ot->start_index = md->index;
   ot->scale_level_delta = 0;
@@ -314,7 +315,7 @@ markup_data_close_tag (MarkupData *md)
     }
   
   g_slist_free (ot->attrs);
-  g_free (ot);  
+  g_slice_free (OpenTag, ot);  
 }
 
 static void
@@ -435,7 +436,6 @@ text_handler           (GMarkupParseContext *context,
       const gchar *end;
       const gchar *range_start;
       const gchar *range_end;
-      gboolean just_saw_marker;
       gssize uline_index = -1;
       gsize uline_len = 0;	/* Quiet GCC */
       
@@ -443,7 +443,6 @@ text_handler           (GMarkupParseContext *context,
       range_start = text;
       p = text;
       end = text + text_len;
-      just_saw_marker = FALSE;
       
       while (p != end)
         {
@@ -552,25 +551,25 @@ static const GMarkupParser pango_markup_parser = {
  * @markup_text: markup to parse (see <link linkend="PangoMarkupFormat">markup format</link>)
  * @length: length of @markup_text, or -1 if nul-terminated
  * @accel_marker: character that precedes an accelerator, or 0 for none
- * @attr_list: address of return location for a #PangoAttrList, or NULL
- * @text: address of return location for text with tags stripped, or NULL
- * @accel_char: address of return location for accelerator char, or NULL
- * @error: address of return location for errors, or NULL
+ * @attr_list: address of return location for a #PangoAttrList, or %NULL
+ * @text: address of return location for text with tags stripped, or %NULL
+ * @accel_char: address of return location for accelerator char, or %NULL
+ * @error: address of return location for errors, or %NULL
  * 
  *
  * Parses marked-up text (see
  * <link linkend="PangoMarkupFormat">markup format</link>) to create
- * a plaintext string and an attribute list.
+ * a plain-text string and an attribute list.
  *
  * If @accel_marker is nonzero, the given character will mark the
- * character following it as an accelerator. For example, the accel
- * marker might be an ampersand or underscore. All characters marked
+ * character following it as an accelerator. For example, @accel_marker
+ * might be an ampersand or underscore. All characters marked
  * as an accelerator will receive a %PANGO_UNDERLINE_LOW attribute,
  * and the first character so marked will be returned in @accel_char.
  * Two @accel_marker characters following each other produce a single
  * literal @accel_marker character.
  * 
- * Return value: FALSE if @error is set, otherwise TRUE
+ * Return value: %FALSE if @error is set, otherwise %TRUE
  **/
 gboolean
 pango_parse_markup (const char                 *markup_text,
@@ -590,7 +589,7 @@ pango_parse_markup (const char                 *markup_text,
   
   g_return_val_if_fail (markup_text != NULL, FALSE);
   
-  md = g_new (MarkupData, 1);
+  md = g_slice_new (MarkupData);
 
   /* Don't bother creating these if they weren't requested;
    * might be useful e.g. if you just want to validate
@@ -601,7 +600,7 @@ pango_parse_markup (const char                 *markup_text,
   else
     md->attr_list = NULL;
 
-  md->text = g_string_new ("");
+  md->text = g_string_new (NULL);
   
   if (accel_char)
     *accel_char = 0;
@@ -685,7 +684,7 @@ pango_parse_markup (const char                 *markup_text,
   
   g_assert (md->tag_stack == NULL);
   
-  g_free (md);
+  g_slice_free (MarkupData, md);
 
   return TRUE;
 
@@ -699,7 +698,7 @@ pango_parse_markup (const char                 *markup_text,
   if (md->attr_list)
     pango_attr_list_unref (md->attr_list);
 
-  g_free (md);
+  g_slice_free (MarkupData, md);
 
   if (context)
     g_markup_parse_context_free (context);
@@ -998,7 +997,7 @@ span_parse_func     (MarkupData            *md,
 
           n = strtoul (size, &end, 10);
 
-          if (*end != '\0')
+          if (*end != '\0' || n < 0 || n > 1000000)
             {
               g_set_error (error,
                            G_MARKUP_ERROR,
