@@ -23,7 +23,6 @@
 #include <stdlib.h>
 
 #include "pango-renderer.h"
-#include "pango-layout-private.h"
 
 #define N_RENDER_PARTS 4
 
@@ -177,7 +176,7 @@ pango_renderer_draw_layout (PangoRenderer    *renderer,
       PangoLayoutLine *line;
       int              baseline;
       
-      line = _pango_layout_iter_get_line_readonly (iter);
+      line = pango_layout_iter_get_line_readonly (iter);
       
       pango_layout_iter_get_line_extents (iter, NULL, &logical_rect);
       baseline = pango_layout_iter_get_baseline (iter);
@@ -482,7 +481,7 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
       PangoRectangle ink_rect, *ink = NULL;
       PangoRectangle logical_rect, *logical = NULL;
 
-      if (run->item->analysis.centered_baseline)
+      if (run->item->analysis.flags & PANGO_ANALYSIS_FLAG_CENTERED_BASELINE)
         logical = &logical_rect;
 
       pango_renderer_prepare_run (renderer, run);
@@ -516,7 +515,7 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
 
       state.logical_rect_end = x + x_off + glyph_string_width;
 
-      if (run->item->analysis.centered_baseline)
+      if (run->item->analysis.flags & PANGO_ANALYSIS_FLAG_CENTERED_BASELINE)
         rise += logical_rect.y + logical_rect.height / 2;
 
 
@@ -1112,7 +1111,7 @@ pango_renderer_get_color (PangoRenderer   *renderer,
  * @renderer: a #PangoRenderer
  * @part: the part for which rendering has changed.
  * 
- * Informs Pango that the way that the renderering is done
+ * Informs Pango that the way that the rendering is done
  * for @part has changed in a way that would prevent multiple
  * pieces being joined together into one drawing call. For
  * instance, if a subclass of #PangoRenderer was to add a stipple
@@ -1147,7 +1146,7 @@ pango_renderer_part_changed (PangoRenderer    *renderer,
  * @renderer: a #PangoRenderer
  * @run: a #PangoLayoutRun
  * 
- * Set up the state of the PangoRenderer for rendering @run.
+ * Set up the state of the #PangoRenderer for rendering @run.
  *
  * Since: 1.8
  **/
