@@ -22,26 +22,24 @@
 #ifndef __PANGO_H__
 #define __PANGO_H__
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
 #include <pango/pango-attributes.h>
 #include <pango/pango-break.h>
 #include <pango/pango-context.h>
 #include <pango/pango-coverage.h>
 #include <pango/pango-engine.h>
 #include <pango/pango-enum-types.h>
+#include <pango/pango-features.h>
 #include <pango/pango-font.h>
 #include <pango/pango-fontmap.h>
+#include <pango/pango-fontset.h>
 #include <pango/pango-glyph.h>
+#include <pango/pango-glyph-item.h>
 #include <pango/pango-item.h>
 #include <pango/pango-layout.h>
+#include <pango/pango-renderer.h>
+#include <pango/pango-script.h>
+#include <pango/pango-tabs.h>
 #include <pango/pango-types.h>
-
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
-
+#include <pango/pango-utils.h>
 
 #endif /* __PANGO_H__ */
