@@ -24,6 +24,7 @@
 #define __PANGOFT2_PRIVATE_H__
 
 #include <pango/pangoft2.h>
+#include <pango/pangofc-fontmap.h>
 #include <pango/pango-renderer.h>
 #include <fontconfig/fontconfig.h>
 
@@ -82,6 +83,8 @@ PangoFT2Font * _pango_ft2_font_new                (PangoFT2FontMap   *ft2fontmap
 						   FcPattern         *pattern);
 FT_Library     _pango_ft2_font_map_get_library    (PangoFontMap      *fontmap);
 const char    *_pango_ft2_ft_strerror             (FT_Error           error);
+void _pango_ft2_font_map_default_substitute (PangoFcFontMap *fcfontmap,
+					     FcPattern      *pattern);
 
 void *_pango_ft2_font_get_cache_glyph_data    (PangoFont      *font,
 					       int             glyph_index);
