@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
+#include <config.h>
 
 #include <string.h>
 #include <stdlib.h>
@@ -33,6 +33,8 @@
 #include "pangofc-fontmap.h"
 #include "pangofc-private.h"
 
+PangoFT2WarningHistory _pango_ft2_warning_history = { FALSE };
+
 /* for compatibility with older freetype versions */
 #ifndef FT_LOAD_TARGET_MONO
 #define FT_LOAD_TARGET_MONO  FT_LOAD_MONOCHROME
@@ -58,8 +60,6 @@ static void                   pango_ft2_font_get_glyph_extents (PangoFont      *
 
 static FT_Face    pango_ft2_font_real_lock_face         (PangoFcFont      *font);
 static void       pango_ft2_font_real_unlock_face       (PangoFcFont      *font);
-static PangoGlyph pango_ft2_font_real_get_unknown_glyph (PangoFcFont      *font,
-							 gunichar          wc);
 
 PangoFT2Font *
 _pango_ft2_font_new (PangoFT2FontMap *ft2fontmap,
@@ -100,6 +100,8 @@ load_fallback_face (PangoFT2Font *ft2font,
 			 FC_PIXEL_SIZE, FcTypeDouble, (double)ft2font->size / PANGO_SCALE,
 			 NULL);
   
+  _pango_ft2_font_map_default_substitute (fcfont->fontmap, sans);
+
   matched = FcFontMatch (NULL, sans, &result);
   
   if (FcPatternGetString (matched, FC_FILE, 0, &filename2) != FcResultMatch)
@@ -153,14 +155,14 @@ set_transform (PangoFT2Font *ft2font)
  * pango_ft2_font_get_face:
  * @font: a #PangoFont
  * 
- * Returns the native FreeType2 FT_Face structure used for this PangoFont.
+ * Returns the native FreeType2 <type>FT_Face</type> structure used for this #PangoFont.
  * This may be useful if you want to use FreeType2 functions directly.
  *
  * Use pango_fc_font_lock_face() instead; when you are done with a
  * face from pango_fc_font_lock_face() you must call
  * pango_fc_font_unlock_face().
  * 
- * Return value: a pointer to a #FT_Face structure, with the size set correctly
+ * Return value: a pointer to a <type>FT_Face</type> structure, with the size set correctly
  **/
 FT_Face
 pango_ft2_font_get_face (PangoFont *font)
@@ -173,6 +175,16 @@ pango_ft2_font_get_face (PangoFont *font)
   FcBool antialias, hinting, autohint;
   int id;
 
+  if (G_UNLIKELY (!PANGO_FT2_IS_FONT (font)))
+    {
+      if (!_pango_ft2_warning_history.get_face)
+        {
+	  _pango_ft2_warning_history.get_face = TRUE;
+	  g_warning ("pango_ft2_font_get_face called with bad font, expect ugly output");
+	}
+      return NULL;
+    }
+
   pattern = fcfont->font_pattern;
 
   if (!ft2font->face)
@@ -259,7 +271,6 @@ pango_ft2_font_class_init (PangoFT2FontClass *class)
   
   fc_font_class->lock_face = pango_ft2_font_real_lock_face;
   fc_font_class->unlock_face = pango_ft2_font_real_unlock_face;
-  fc_font_class->get_unknown_glyph = pango_ft2_font_real_get_unknown_glyph;
 }
 
 static PangoFT2GlyphInfo *
@@ -275,7 +286,7 @@ pango_ft2_font_get_glyph_info (PangoFont   *font,
 
   if ((info == NULL) && create)
     {
-       info = g_new0 (PangoFT2GlyphInfo, 1);
+      info = g_slice_new0 (PangoFT2GlyphInfo);
 
       pango_fc_font_get_raw_extents (fcfont, ft2font->load_flags,
 				     glyph,
@@ -296,6 +307,53 @@ pango_ft2_font_get_glyph_extents (PangoFont      *font,
 {
   PangoFT2GlyphInfo *info;
 
+  if (glyph == PANGO_GLYPH_EMPTY)
+    {
+      if (ink_rect)
+	ink_rect->x = ink_rect->y = ink_rect->height = ink_rect->width = 0;
+      if (logical_rect)
+	logical_rect->x = logical_rect->y = logical_rect->height = logical_rect->width = 0;
+      return;
+    }
+
+  if (glyph & PANGO_GLYPH_UNKNOWN_FLAG)
+    {
+      glyph = pango_ft2_get_unknown_glyph (font);
+      if (glyph == PANGO_GLYPH_EMPTY)
+        {
+	  /* No unknown glyph found for the font, draw a box */
+	  PangoFontMetrics *metrics = pango_font_get_metrics (font, NULL);
+
+	  if (metrics)
+	    {
+	      if (ink_rect)
+		{
+		  ink_rect->x = PANGO_SCALE;
+		  ink_rect->width = metrics->approximate_char_width - 2 * PANGO_SCALE;
+		  ink_rect->y = - (metrics->ascent - PANGO_SCALE);
+		  ink_rect->height = metrics->ascent + metrics->descent - 2 * PANGO_SCALE;
+		}
+	      if (logical_rect)
+		{
+		  logical_rect->x = 0;
+		  logical_rect->width = metrics->approximate_char_width;
+		  logical_rect->y = -metrics->ascent;
+		  logical_rect->height = metrics->ascent + metrics->descent;
+		}
+
+	      pango_font_metrics_unref (metrics);
+	    }
+	  else
+	    {
+	      if (ink_rect)
+		ink_rect->x = ink_rect->y = ink_rect->height = ink_rect->width = 0;
+	      if (logical_rect)
+		logical_rect->x = logical_rect->y = logical_rect->height = logical_rect->width = 0;
+	    }
+	  return;
+	}
+    }
+
   info = pango_ft2_font_get_glyph_info (font, glyph, TRUE);
 
   if (ink_rect)
@@ -360,13 +418,6 @@ pango_ft2_font_real_unlock_face (PangoFcFont *font)
 {
 }
 
-static PangoGlyph
-pango_ft2_font_real_get_unknown_glyph (PangoFcFont *font,
-				       gunichar     wc)
-{
-  return 0;
-}
-
 static gboolean
 pango_ft2_free_glyph_info_callback (gpointer key,
 				    gpointer value,
@@ -378,7 +429,7 @@ pango_ft2_free_glyph_info_callback (gpointer key,
   if (font->glyph_cache_destroy && info->cached_glyph)
     (*font->glyph_cache_destroy) (info->cached_glyph);
 
-  g_free (value);
+  g_slice_free (PangoFT2GlyphInfo, info);
   return TRUE;
 }
 
@@ -421,22 +472,30 @@ pango_ft2_font_get_coverage (PangoFont     *font,
  * pango_ft2_get_unknown_glyph:
  * @font: a #PangoFont
  * 
- * Return the index of a glyph suitable for drawing unknown characters.
+ * Return the index of a glyph suitable for drawing unknown characters with
+ * @font, or %PANGO_GLYPH_EMPTY if no suitable glyph found.
  * 
- * Use pango_fc_font_get_unknown_glyph() instead.
+ * If you want to draw an unknown-box for a character that is not covered
+ * by the font,
+ * use PANGO_GET_UNKNOWN_GLYPH() instead.
  * 
- * Return value: a glyph index into @font
+ * Return value: a glyph index into @font, or %PANGO_GLYPH_EMPTY
  **/
 PangoGlyph
 pango_ft2_get_unknown_glyph (PangoFont *font)
 {
-  return 0;
+  FT_Face face = pango_ft2_font_get_face (font);
+  if (face && FT_IS_SFNT (face))
+    /* TrueType fonts have an 'unknown glyph' box on glyph index 0 */
+    return 0;
+  else
+    return PANGO_GLYPH_EMPTY;
 }
 
 typedef struct
 {
   FT_Error     code;
-  const char*  msg;
+  const char   msg[40];
 } ft_error_description;
 
 static int
@@ -446,15 +505,15 @@ ft_error_compare (const void *pkey,
   return ((ft_error_description *) pkey)->code - ((ft_error_description *) pbase)->code;
 }
 
-const char *
+G_CONST_RETURN char *
 _pango_ft2_ft_strerror (FT_Error error)
 {
 #undef __FTERRORS_H__
 #define FT_ERRORDEF( e, v, s )  { e, s },
 #define FT_ERROR_START_LIST  {
-#define FT_ERROR_END_LIST    { 0, 0 } };
+#define FT_ERROR_END_LIST    };
 
-  const ft_error_description ft_errors[] =
+  static const ft_error_description ft_errors[] =
 #include FT_ERRORS_H
 
 #undef FT_ERRORDEF
@@ -462,7 +521,7 @@ _pango_ft2_ft_strerror (FT_Error error)
 #undef FT_ERROR_END_LIST
 
   ft_error_description *found =
-    bsearch (&error, ft_errors, G_N_ELEMENTS (ft_errors) - 1,
+    bsearch (&error, ft_errors, G_N_ELEMENTS (ft_errors),
 	     sizeof (ft_errors[0]), ft_error_compare);
   if (found != NULL)
     return found->msg;
@@ -484,7 +543,8 @@ _pango_ft2_font_get_cache_glyph_data (PangoFont *font,
 {
   PangoFT2GlyphInfo *info;
 
-  g_return_val_if_fail (PANGO_FT2_IS_FONT (font), NULL);
+  if (!PANGO_FT2_IS_FONT (font))
+    return NULL;
   
   info = pango_ft2_font_get_glyph_info (font, glyph_index, FALSE);
 
@@ -501,7 +561,8 @@ _pango_ft2_font_set_cache_glyph_data (PangoFont     *font,
 {
   PangoFT2GlyphInfo *info;
 
-  g_return_if_fail (PANGO_FT2_IS_FONT (font));
+  if (!PANGO_FT2_IS_FONT (font))
+    return;
   
   info = pango_ft2_font_get_glyph_info (font, glyph_index, TRUE);
 
@@ -514,7 +575,8 @@ void
 _pango_ft2_font_set_glyph_cache_destroy (PangoFont      *font,
 					 GDestroyNotify  destroy_notify)
 {
-  g_return_if_fail (PANGO_FT2_IS_FONT (font));
+  if (!PANGO_FT2_IS_FONT (font))
+    return;
   
   PANGO_FT2_FONT (font)->glyph_cache_destroy = destroy_notify;
 }
